# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.core.management.base import CommandError
from django.core.management.color import no_style
from django.core.management.commands.flush import Command as FlushCommand

from identityprovider.utils import is_django_13


class Command(FlushCommand):
    cached_flush = None

    def handle_noargs(self, **options):
        from django.conf import settings
        from django.core.management.sql import sql_flush, emit_post_sync_signal
        from django.db import connection, transaction, models

        verbosity = int(options.get('verbosity', 1))
        interactive = options.get('interactive')

        self.style = no_style()

        if self.cached_flush is not None:
            try:
                cursor = connection.cursor()
                cursor.execute(self.cached_flush)
            except Exception, e:
                transaction.rollback_unless_managed()
                raise CommandError("""Database %s couldn't be flushed. Possible reasons:
      * The database isn't running or isn't configured correctly.
      * At least one of the expected database tables doesn't exist.
      * The SQL was invalid.
    Hint: Look at the output of 'django-admin.py sqlflush'. That's the SQL this command wasn't able to run.
    The full error: %s""" % (settings.DATABASE_NAME, e))
            transaction.commit_unless_managed()

            # Emit the post sync signal. This allows individual
            # applications to respond as if the database had been
            # sync'd from scratch.
            emit_post_sync_signal(models.get_models(), verbosity, interactive)

            # Reinstall the initial_data fixture.
            from django.core.management import call_command
            call_command('loaddata', 'initial_data', **options)
        else:
            super(Command, self).handle_noargs(**options)
            # Call sql_flush instead of picking up the raw queries because
            # it's better SQL.
            args = [self.style]
            if is_django_13():
                args.append(connection)

            queries = sql_flush(*args, only_django=True)
            Command.cached_flush = ';'.join(queries)
