# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import signal
from subprocess import Popen, PIPE

from django.conf import settings
from django.core.management import call_command

from identityprovider.tests.utils import SQLCachedTestCase


class Alarm(Exception):
    pass


def alarm_handler(signum, frame):
    raise Alarm


class SQLCachedFlushCommandTestCase(SQLCachedTestCase):
    def setUp(self):
        # set alarm handler
        self.old_signal = signal.signal(signal.SIGALRM, alarm_handler)

    def tearDown(self):
        # reset alarm handler
        signal.signal(signal.SIGALRM, self.old_signal)

    def test_sqlcachedflush_command(self):
        args = ['pg_dump']
        if settings.DATABASE_USER:
            args.extend(['-U', settings.DATABASE_USER])
        if settings.DATABASE_PASSWORD:
            _PGPASS = os.environ.get('PGPASSWORD')
            os.environ['PGPASSWORD'] = settings.DATABASE_PASSWORD
        if settings.DATABASE_HOST:
            args.extend(['-h', settings.DATABASE_HOST])
        if settings.DATABASE_PORT:
            args.extend(['-p', settings.DATABASE_PORT])
        args.append(settings.DATABASE_NAME)

        call_command('flush', interactive=False)
        flush_sql = Popen(args, stdout=PIPE).communicate()[0]

        call_command('sqlcachedflush', interactive=False)
        # timeout after 5 seconds
        signal.alarm(5)
        try:
            sqlcachedflush_sql = Popen(args, stdout=PIPE).communicate()[0]
            # reset the alarm
            signal.alarm(0)
        except Alarm:
            self.fail()

        self.assertEqual(sqlcachedflush_sql, flush_sql)

