# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.core import mail

from identityprovider.branding import current_brand
from identityprovider.models.authtoken import AuthToken
from utils import BasicAccountTestCase, SQLCachedTestCase


class LoginTest(SQLCachedTestCase):

    def test_loginform_password_did_not_match_error(self):
        r = self.client.post('/+login', {'email': "mark@example.com",
                                         'password': "foo"})
        self.assertFormError(r, 'form', None, "Password didn't match.")

    def test_loginform_wrong_email_error(self):
        r = self.client.post('/+login', {'email': "foo@"})
        self.assertFormError(r, 'form', 'email', "Invalid email.")

    def test_loginform_password_required_error(self):
        r = self.client.post('/+login', {'email': "mark@example.com",
                                         'passwrd': ""})
        self.assertFormError(r, 'form', 'password', "Required field.")


class NewAccountTest(BasicAccountTestCase):

    def test_newaccount_existing(self):
        query = {
            'email': 'nobody@debian.org',
        }
        response = self.client.post('/+forgot_password', query)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['location'],
                          'http://testserver/+email-sent')
        self.assertEquals(len(mail.outbox), 0)


class ForgottenPasswordTest(BasicAccountTestCase):

    def test_forgottenpass_nonexisting(self):
        query = {
            'displayname': 'Tester',
            'email': 'test@canonical.com',
            'password': 'Testing123',
            'passwordconfirm': 'Testing123'
        }
        response = self.client.post('/+new_account', query)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['location'],
                          'http://testserver/+email-sent')
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(mail.outbox[0].subject,
            u"%s: Warning" % current_brand.name)
        mail.outbox = []

    def test_forgottenform_success(self):
        query = {
            'email': 'test@canonical.com',
        }
        response = self.client.post('/+forgot_password', query)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response['location'],
                          'http://testserver/+email-sent')
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(mail.outbox[0].subject,
            u"%s: Forgotten Password" % current_brand.name)
        mail.outbox = []

        AuthToken.objects.all().delete()
