# Copyright 2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import

import os
import sys

from fabric.api import (
    env,
    local,
)


__all__ = [
    'install_dependencies',
    'setup_virtualenv',
    'virtualenv',
    ]

VIRTUALENV = '.env'


def virtualenv(command, capture=True):
    """Run a command inside the virtualenv."""
    prefix = ''
    virtual_env = env.get('virtualenv', None)
    if virtual_env:
        prefix = ". %s/bin/activate && " % virtual_env
    command = prefix + command
    return local(command, capture=capture)


def setup_virtualenv():
    """Setup the virtualenv."""
    created = False
    virtual_env = os.environ.get('VIRTUAL_ENV', None)
    if virtual_env is None:
        if not os.path.exists(VIRTUALENV):
            _create_virtualenv()
            created = True
        virtual_env = VIRTUALENV
    env.virtualenv = os.path.abspath(virtual_env)
    _activate_virtualenv()
    return created


def install_dependencies():
    """Install all dependencies into the virtualenv."""
    cmd = ['pip', 'install', '-r', 'requirements.txt']
    virtualenv(' '.join(cmd), capture=False)
    virtualenv('python setup.py develop')


# helpers
# =======


def _activate_virtualenv():
    """Activate the virtualenv."""
    activate_this = os.path.abspath("%s/bin/activate_this.py" % env.virtualenv)
    execfile(activate_this, dict(__file__=activate_this))

def _create_virtualenv(clear=False):
    """Create the virtualenv."""
    if not os.path.exists(VIRTUALENV) or clear:
        args = '--no-site-packages --distribute --clear'
        local("%s /usr/bin/virtualenv %s %s" % (sys.executable, args, VIRTUALENV), capture=False)

